@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2014	PgPool Global Development Group
+ * Copyright (c) 2003-2012	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -34,7 +34,7 @@ typedef struct {
 	bool	has_temp_table;		/* True if temporary table is used */
 	bool	has_unlogged_table;	/* True if unlogged table is used */
 	bool	has_function_call;	/* True if write function call is used */	
- 	bool	has_insertinto_or_locking_clause;	/* True if it has SELECT INTO or FOR SHARE/UPDATE */
+	bool	has_insertinto_or_locking_clause;	/* True if it has SELECT INTO or FOR SHARE/UPDATE */
 } SelectContext;
 
 static bool function_call_walker(Node *node, void *context);
@@ -45,13 +45,15 @@ static bool unlogged_table_walker(Node *node, void *context);
 static bool is_temp_table(char *table_name);
 static bool is_unlogged_table(char *table_name);
 static bool	insertinto_or_locking_clause_walker(Node *node, void *context);
+static int pattern_compare(char *str, const int type);
 
 /*
- * Return true if this SELECT has function calls.
+ * Return true if this SELECT has function calls *and* supposed to
+ * modify database.  We check black/white function list to determine
+ * whether the function modifies database.
  */
 bool pool_has_function_call(Node *node)
 {
-
 	SelectContext	ctx;
 
 	if (!IsA(node, SelectStmt))
@@ -118,7 +120,7 @@ bool pool_has_unlogged_table(Node *node)
 	return ctx.has_unlogged_table;
 }
 
- /*
+/*
  * Return true if this SELECT has INSERT INTO or FOR SHARE or FOR UDPATE.
  */
 bool pool_has_insertinto_or_locking_clause(Node *node)
@@ -139,12 +141,48 @@ bool pool_has_insertinto_or_locking_clause(Node *node)
 }
 
 /*
- * Walker function to find a function call
+ * Search function name in whilelist or blacklist regex array
+ * Return 1 on success (found in list)
+ * Return 0 when not found in list
+ * Return -1 if the given search type doesn't exist.
+ * Search type supported are: WHITELIST and BLACKLIST 
+ */
+static int pattern_compare(char *str, const int type)
+{
+	int i = 0;
+
+	/* pass througth all regex pattern unless pattern is found */
+	for (i = 0; i < pool_config->pattc; i++) {
+		if ( (pool_config->lists_patterns[i].type == type) && (regexec(&pool_config->lists_patterns[i].regexv, str, 0, 0, 0) == 0) ) {
+			switch(type) {
+			/* return 1 if string matches whitelist pattern */
+			case WHITELIST:
+				if (pool_config->debug_level > 0)
+					pool_debug("pattern_compare: white_function_list (%s) matched: %s", pool_config->lists_patterns[i].pattern, str);
+				return 1;
+			/* return 1 if string matches blacklist pattern */
+			case BLACKLIST:
+				if (pool_config->debug_level > 0)
+					pool_debug("pattern_compare: black_function_list (%s) matched: %s", pool_config->lists_patterns[i].pattern, str);
+				return 1;
+			default:
+				pool_error("pattern_compare: unknown pattern match type: %s", str);
+				return -1;
+			}
+		}
+	}
+
+	/* return 0 otherwise */
+	return 0;
+}
+
+/*
+ * Walker function to find a function call which is supposed to write
+ * database.
  */
 static bool function_call_walker(Node *node, void *context)
 {
 	SelectContext	*ctx = (SelectContext *) context;
-	int i;
 
 	if (node == NULL)
 		return false;
@@ -173,13 +211,10 @@ static bool function_call_walker(Node *node, void *context)
 			 */
 			if (pool_config->num_white_function_list > 0)
 			{
-				for (i=0;i<pool_config->num_white_function_list;i++)
-				{
+				/* Search function in the white list regex patterns */
+				if (pattern_compare(fname, WHITELIST) == 1) {
 					/* If the function is found in the white list, we can ignore it */
-					if (!strcasecmp(pool_config->white_function_list[i], fname))
-					{
-						return raw_expression_tree_walker(node, function_call_walker, context);
-					}
+					return raw_expression_tree_walker(node, function_call_walker, context);
 				}
 				/*
 				 * Since the function was not found in white list, we
@@ -192,11 +227,10 @@ static bool function_call_walker(Node *node, void *context)
 			/*
 			 * Check black list if any.
 			 */
-			for (i=0;i<pool_config->num_black_function_list;i++)
+			if (pool_config->num_black_function_list > 0)
 			{
-				/* Is the function found in the black list? */
-				if (!strcasecmp(pool_config->black_function_list[i], fname))
-				{
+				/* Search function in the black list regex patterns */
+				if (pattern_compare(fname, BLACKLIST) == 1) {
 					/* Found. */
 					ctx->has_function_call = true;
 					return false;
@@ -541,7 +575,7 @@ static bool is_unlogged_table(char *table_name)
 		{
 			relcache = pool_create_relcache(128, query,
 											int_register_func, int_unregister_func,
-											false);
+											true);
 			if (relcache == NULL)
 			{
 				pool_error("is_unlogged_table: pool_create_relcache error");
@@ -569,19 +603,16 @@ bool pool_has_pgpool_regclass(void)
 /*
  * Query to know if pgpool_regclass exists.
  */
-#define HASPGPOOL_REGCLASSQUERY "SELECT count(*) FROM pg_catalog.pg_proc AS p WHERE p.proname = 'pgpool_regclass' AND has_function_privilege('%s', 'pgpool_regclass(cstring)', 'execute')"
-
+#define HASPGPOOL_REGCLASSQUERY "SELECT count(*) FROM pg_catalog.pg_proc AS p WHERE p.proname = '%s'"
 	bool result;
 	static POOL_RELCACHE *relcache;
 	POOL_CONNECTION_POOL *backend;
-	char *user;
 
 	backend = pool_get_session_context()->backend;
-	user = MASTER_CONNECTION(backend)->sp->user;
 
 	if (!relcache)
 	{
-		relcache = pool_create_relcache(128, HASPGPOOL_REGCLASSQUERY,
+		relcache = pool_create_relcache(32, HASPGPOOL_REGCLASSQUERY,
 										int_register_func, int_unregister_func,
 										false);
 		if (relcache == NULL)
@@ -591,7 +622,7 @@ bool pool_has_pgpool_regclass(void)
 		}
 	}
 
-	result = pool_search_relcache(relcache, backend, user)==0?0:1;
+	result = pool_search_relcache(relcache, backend, "pgpool_regclass")==0?0:1;
 	return result;
 }
 
